@@ -18,6 +18,7 @@
    limitations under the License.
 */
 #include "mainwindow.h"
+#include "passedit.h"
 
 #include <QDebug>
 #include <QFileDialog>
@@ -40,6 +41,8 @@ MainWindow::MainWindow(QWidget *parent)
     setupUi(this);
     setWindowFlags(Qt::Window); // for the close, min and max buttons
     setWindowIcon(QApplication::windowIcon());
+    passUser = new PassEdit(userPasswordEdit, userPassword2Edit, 2, this);
+    passChange = new PassEdit(lineEditChangePass, lineEditChangePassConf, 2, this);
 
     shell = new Cmd(this);
     tabWidget->blockSignals(true);
@@ -57,7 +60,10 @@ MainWindow::MainWindow(QWidget *parent)
     refresh();
 }
 
-MainWindow::~MainWindow() { settings.setValue(QStringLiteral("geometry"), saveGeometry()); }
+MainWindow::~MainWindow()
+{
+    settings.setValue(QStringLiteral("geometry"), saveGeometry());
+}
 
 void MainWindow::refresh()
 {
@@ -139,6 +145,7 @@ void MainWindow::refreshAdd()
     userPasswordEdit->clear();
     userPassword2Edit->clear();
     addUserBox->setEnabled(true);
+    checkSudoGroup->setVisible(shell->run("grep -q '^EXTRA_GROUPS=.*\\<sudo\\>' /etc/adduser.conf", true));
 }
 
 void MainWindow::refreshDelete()
@@ -333,11 +340,11 @@ void MainWindow::applyAdd()
         QMessageBox::critical(this, windowTitle(), tr("Sorry, this name is in use. Please enter a different name."));
         return;
     }
-    if (userPasswordEdit->text() != userPassword2Edit->text()) {
+    if (!passUser->confirmed()) {
         QMessageBox::critical(this, windowTitle(), tr("Password entries do not match. Please try again."));
         return;
     }
-    if (userPasswordEdit->text().length() < 2) {
+    if (!passUser->lengthOK()) {
         QMessageBox::critical(
             this, windowTitle(),
             tr("Password needs to be at least 2 characters long. Please enter a longer password before proceeding."));
@@ -360,8 +367,8 @@ void MainWindow::applyAdd()
     if (allowBadNames != QStringLiteral("--allow-bad-names"))
         allowBadNames = QStringLiteral("--force-badname");
 
-    QProcess::execute("adduser", {"--disabled-login", allowBadNames, "--shell", dshell,
-                                  commentOption, userNameEdit->text(), userNameEdit->text()});
+    QProcess::execute("adduser", {"--disabled-login", allowBadNames, "--shell", dshell, commentOption,
+                                  userNameEdit->text(), userNameEdit->text()});
 
     QProcess proc;
     proc.start(QStringLiteral("passwd"), QStringList {userNameEdit->text()}, QIODevice::ReadWrite);
@@ -371,6 +378,8 @@ void MainWindow::applyAdd()
     proc.waitForFinished();
 
     if (proc.exitCode() == 0) {
+        if (!checkSudoGroup->isChecked())
+            QProcess::execute("delgroup", {userNameEdit->text(), "sudo"});
         QMessageBox::information(this, windowTitle(), tr("The user was added ok."));
         refresh();
     } else {
@@ -381,11 +390,11 @@ void MainWindow::applyAdd()
 // change user password
 void MainWindow::applyChangePass()
 {
-    if (lineEditChangePass->text() != lineEditChangePassConf->text()) {
+    if (!passChange->confirmed()) {
         QMessageBox::critical(this, windowTitle(), tr("Password entries do not match. Please try again."));
         return;
     }
-    if (lineEditChangePass->text().length() < 2) {
+    if (!passChange->lengthOK()) {
         QMessageBox::critical(
             this, windowTitle(),
             tr("Password needs to be at least 2 characters long. Please enter a longer password before proceeding."));
@@ -478,13 +487,14 @@ void MainWindow::applyGroup()
                         .arg(groups.join(" "));
         int ans = QMessageBox::warning(this, windowTitle(), msg, QMessageBox::Yes, QMessageBox::No);
         if (ans == QMessageBox::Yes) {
-            for (const auto &group : qAsConst(groups)) {
-                if (QProcess::execute("delgroup", {group}) != 0) {
-                    QMessageBox::critical(this, windowTitle(),
-                                          tr("Failed to delete the group.") + "\n" + tr("Group: %1").arg(group));
-                    refresh();
-                    return;
-                }
+            auto it = std::find_if(groups.cbegin(), groups.cend(),
+                                   [&](const auto &group) { return QProcess::execute("delgroup", {group}) != 0; });
+            if (it != groups.cend()) {
+                const auto &group = *it;
+                QMessageBox::critical(this, windowTitle(),
+                                      tr("Failed to delete the group.") + "\n" + tr("Group: %1").arg(group));
+                refresh();
+                return;
             }
             msg = groups.count() == 1 ? tr("The group has been deleted.") : tr("The groups have been deleted.");
             QMessageBox::information(this, windowTitle(), msg);
@@ -929,42 +939,6 @@ void MainWindow::on_sharedRadioButton_clicked()
     syncProgressBar->setValue(0);
 }
 
-void MainWindow::on_userPassword2Edit_textChanged(const QString &arg1)
-{
-    QPalette pal = userPassword2Edit->palette();
-    if (arg1 != userPasswordEdit->text())
-        pal.setColor(QPalette::Base, QColor(255, 0, 0, 20));
-    else
-        pal.setColor(QPalette::Base, QColor(0, 255, 0, 10));
-    userPasswordEdit->setPalette(pal);
-    userPassword2Edit->setPalette(pal);
-}
-
-void MainWindow::on_lineEditChangePassConf_textChanged(const QString &arg1)
-{
-    QPalette pal = lineEditChangePassConf->palette();
-    if (arg1 != lineEditChangePass->text())
-        pal.setColor(QPalette::Base, QColor(255, 0, 0, 20));
-    else
-        pal.setColor(QPalette::Base, QColor(0, 255, 0, 10));
-    lineEditChangePassConf->setPalette(pal);
-    lineEditChangePass->setPalette(pal);
-}
-
-void MainWindow::on_userPasswordEdit_textChanged()
-{
-    userPassword2Edit->clear();
-    userPasswordEdit->setPalette(QApplication::palette());
-    userPassword2Edit->setPalette(QApplication::palette());
-}
-
-void MainWindow::on_lineEditChangePass_textChanged()
-{
-    lineEditChangePassConf->clear();
-    lineEditChangePass->setPalette(QApplication::palette());
-    lineEditChangePassConf->setPalette(QApplication::palette());
-}
-
 void MainWindow::on_comboRenameUser_activated(const QString & /*unused*/)
 {
     addUserBox->setEnabled(false);
